;;; Copyright (C) 2001 Ian Campbell <ijc@hellion.org.uk>
;;;
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 2 of the
;;; License, or (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
;;; USA.

(define-structure xmms.command 

    (export xmms-play
	    xmms-play-pause
	    xmms-stop
	    xmms-prev
	    xmms-next
	    xmms-set-playlist-pos
	    xmms-playlist-add
	    xmms-playlist-set
	    xmms-playlist-delete
	    xmms-playlist-clear
	    xmms-jump-to-time
	    xmms-set-volume
	    xmms-set-main-volume
	    xmms-inc-main-volume
	    xmms-dec-main-volume
	    xmms-set-balance
	    xmms-all-show
	    xmms-all-hide
	    xmms-main-show
	    xmms-main-hide
	    xmms-pl-show
	    xmms-pl-hide
	    xmms-eq-show
	    xmms-eq-show
	    xmms-show-prefs
	    xmms-set-always-on-top
	    xmms-unset-always-on-top
	    xmms-eject
	    xmms-add-url
	    xmms-toggle-repeat
	    xmms-toggle-shuffle
	    xmms-quit)

    (open rep 
	  sawfish.wm.commands
	  xmms.remote)
	    
    (define-command 'xmms-play xmms-play)
    (define-command 'xmms-play-pause 
      (lambda ()
	"Toggle Play/Pause."
	(if (xmms-playing-p)
	    (xmms-pause)
	  (xmms-play))))

    (define-command 'xmms-stop xmms-stop)
    (define-command 'xmms-pause xmms-pause)
    (define-command 'xmms-prev xmms-prev)
    (define-command 'xmms-next xmms-next)
    
    (define-command 'xmms-set-playlist-pos xmms-set-playlist-pos
      #:type `(labelled ,(_ "Position:") (number 0)))

    (define-command 'xmms-playlist-add xmms-playlist-add
      #:type `(and (quoted (list string ,(_ "Tracks:")))))

    (define-command 'xmms-playlist-delete xmms-playlist-delete
      #:type `(labelled ,(_ "Position:") (number 0)))

    (define-command 'xmms-playlist-clear xmms-playlist-clear)
    (define-command 'xmms-jump-to-time xmms-jump-to-time
      #:type `(labelled ,(_ "Time:") number))
    
    (define-command 'xmms-set-volume xmms-set-volume
      #:type `(pair (labelled ,(_ "Left Channel:") (number 1 100))
		    (labelled ,(_ "Right Channel:") (number 1 100))))
    
    (define-command 'xmms-set-main-volume xmms-set-main-volume 
      #:type `(labelled ,(_ "Volume:")(number 1 100)))
    
    (define-command 'xmms-inc-main-volume 
      (lambda (step)
	(xmms-set-main-volume (+ (xmms-get-main-volume) step)))
      #:type `(and (labelled ,(_ "Step:")(number 1 100))))

    (define-command 'xmms-dec-main-volume
      (lambda (step)
	(xmms-set-main-volume (- (xmms-get-main-volume) step)))
      #:type `(and (labelled ,(_ "Step:") (number 1 100))))

    (define-command 'xmms-set-balance xmms-set-balance
      #:type `(labelled ,(_ "Balance:") (number -100 100)))

    (define-command 'xmms-all-show
      (lambda () 
	"Show all the windows."
	(xmms-toggle-main t)
	(xmms-toggle-eq t)
	(xmms-toggle-pl t)))

    (define-command 'xmms-all-hide
      (lambda () 
	"Hide all the main window."
	(xmms-toggle-main nil)
	(xmms-toggle-eq nil)
	(xmms-toggle-pl nil)))

    (define-command 'xmms-main-show
      (lambda () 
	"Show the main window."
	(xmms-toggle-main t)))
    (define-command 'xmms-main-hide
      (lambda () 
	"Hide the main window."
	(xmms-toggle-main nil)))
    
    (define-command 'xmms-pl-show 
      (lambda () 
	"Show the playlist window."
	(xmms-toggle-pl t)))
    (define-command 'xmms-pl-hide 
      (lambda () 
	"Hide the playlist window."
	(xmms-toggle-pl nil)))
    
    (define-command 'xmms-eq-show 
      (lambda () 
	"Show the equalizer window."
	(xmms-toggle-eq t)))
    (define-command 'xmms-eq-hide 
      (lambda () 
	"Hide the equalizer window."
	(xmms-toggle-eq nil)))
    
    (define-command 'xmms-prefs-show xmms-prefs-show)
    
    (define-command 'xmms-set-always-on-top
      (lambda () 
	"Make XMMS stay on top of other windows."
	(xmms-toggle-always-on-top t)))
    (define-command 'xmms-unset-always-on-top
      (lambda () 
	"Don't make XMMS stay on top of other windows."
	(xmms-toggle-always-on-top nil)))
    
    (define-command 'xmms-eject xmms-eject)
    (define-command 'xmms-add-url xmms-add-url)
    (define-command 'xmms-toggle-repeat xmms-toggle-repeat)
    (define-command 'xmms-toggle-shuffle xmms-toggle-shuffle)
    (define-command 'xmms-quit xmms-quit)
)